/*

The Z80 Dictionary by Kouri (Kouri@juno.com)		August 6, 1998

	Yay, this is my first C program ever. It's not very complicated;
the hardest part was getting all the text to allign correctly. :)
I made it because I couldn't find any document that gave the size, number
of clock cycles, and a description of the instruction all in one file.

	To use it you simply type "define <instruction>". For example, to
list all the "or" instructions, this is what you would type:

define or

Should you be forgetful of this simple syntax, simply type "define" at the
DOS prompt and you will be reminded. :)

	There are a couple of things wrong with this program however. First,
there are too many "ld" instructions to fit on one screen. The way around
this is to use the "|more" command with the program. Like this:

define ld |more

Not that bad. The second thing is that this program does not cover any of
the undocumented instructions, such as SLL (SL1) or instructions with the
IXH, IXL, IYH, or IYL registers. Please feel free to modify this program to
include them, just tell me if you do so and make sure to release the source.
Lastly (and least of the problems), is that this program will not recognize
instructions typed in CAPS. SO DON'T USE THEM!

	I hope you enjoy this program, and that it is useful to you. I have
absolutely no intention of ever updating this file unless I discover some
horrible, hideous bug or error. If someone would like to make a similar
program for say, the 68K processor, please be my guest. Maybe you will even
see something in this code that you can use. Happy coding!

*/

#include <stdio.h>
#include <string.h>
#define	ENTRIES	160

void main(argc,argv)
	int	argc;
	char	*argv[];
{
	short	c;
	static struct
	{
		char	instruction[5];
		char	operand[14];
		short	size;
		short	clocks;
		char	description[141];
	}
	dictionary[ENTRIES-1]={
	{"adc", "A,<data>",	2, 7,"Add data to accumulator with carry"},
	{"adc", "A,<reg> ",	1, 4,"Add register to accumulator with carry"},
	{"adc", "A,(HL)  ",	1, 7,"Add indirect memory to accumulator with carry"},
	{"adc", "A,(I*)  ",	3,19,"Add indirect memory to accumulator with carry"},
	{"adc", "HL,<rp> ",	2,15,"Add register pair to HL with carry"},

	{"add", "A,<data>",	2, 7,"Add data to accumulator"},
	{"add", "A,<reg> ",	1, 4,"Add register to accumulator"},
	{"add", "A,(HL)  ",	1, 7,"Add indirect memory to accumulator"},
	{"add", "A,(I*)  ",	3,19,"Add indirect memory to accumulator"},
	{"add", "HL,<rp> ",	1,11,"Add register pair to HL"},
	{"add", "I?,<rp> ",	2,15,"Add register pair to index register"},

	{"and", "<data>  ",	2, 7,"And data with accumulator"},
	{"and", "<reg>   ",	1, 4,"And register with accumulator"},
	{"and", "(HL)    ",	1, 7,"And memory with accumulator"},
	{"and", "(I*)    ",	3,19,"And memory with accumulator"},

	{"bit", "<bit>,<reg>",	2, 8,"Test <bit> in register"},
	{"bit", "<b>,(HL)",	2,12,"Test <bit> in indirect memory"},
	{"bit", "<b>,(I*)",	4,20,"Test <bit> in indirect memory"},

	{"call","<addr>  ",	3,17,"Call the routine at <addr>"},
	{"call","<cond>,<addr>",3,17,"Call the routine if condition is satisfied"},

	{"ccf", "NONE    ",	1, 4,"Complement carry flag"},

	{"cp",  "<data>  ",	2, 7,"Compare data with accumulator"},
	{"cp",  "<reg>   ",	1, 4,"Compare register with accumulator"},
	{"cp",  "(HL)    ",	1, 7,"Compare indirect memory with accumulator"},
	{"cp",  "(I*)    ",	3,19,"Compare indirect memory with accumulator"},

	{"cpd", "NONE    ",	2,16,"Compare accumulator with memory and decrement\n\t\t\t\taddress and byte counter"},
	{"cpdr","NONE    ",	2,21,"Compare accumulator with memory and decrement\n\t\t\t\taddress and byte counter; continue until match\n\t\t\t\tis found or byte counter is zero"},
	{"cpi", "NONE    ",	2,16,"Compare accumulator with memory, increment\n\t\t\t\taddress and decrement byte counter"},
	{"cpir","NONE    ",	2,21,"Compare accumulator with memory, increment\n\t\t\t\taddress and decrement byte counter; continue\n\t\t\t\tuntil match is found or byte counter is zero"},

	{"cpl", "NONE    ",	1, 4,"Complement the accumulator"},

	{"daa", "NONE    ",	1, 4,"Decimal adjust accumulator"},

	{"dec", "<reg>   ",	1, 4,"Decrement register contents"},
	{"dec", "<rp>    ",	1, 6,"Decrement register pair contents"},
	{"dec", "I?      ",	2,10,"Decrement index register"},
	{"dec", "(HL)    ",	1,11,"Decrement indirect memory contents"},
	{"dec", "(I*)    ",	3,23,"Decrement indirect memory contents"},

	{"di",  "NONE    ",	1, 4,"Disable interrupts"},

	{"djnz","<addr>  ",	1,13,"Decrement register B and jump relative if zero"},

	{"ei",  "NONE    ",	1, 4,"Enable interrupts"},

	{"ex",  "AF,AF'  ",	1, 4,"Exchange program status and alt program status"},
	{"ex",  "DE,HL   ",	1, 4,"Exchange DE and HL contents"},
	{"ex",  "(SP),HL ",	1,19,"Exchange contents of HL and top of stack"},
	{"ex",  "(SP),I? ",	2,23,"Exchange contents of I? and top of stack"},
	{"exx", "NONE    ",	1, 4,"Exchange register pairs and alt reg pairs"},

	{"halt","NONE    ",	1, 4,"Program execution stops"},

	{"im",  "<n>     ",	2, 8,"Interrupt mode n"},

	{"in",  "A,<port>",	2,11,"Input port to accumulator"},
	{"in",  "<reg>,(C)",	2,12,"Input to register"},

	{"inc", "<reg>   ",	1, 4,"Increment contents of register"},
	{"inc", "<rp>    ",	1, 6,"Increment contents of register pair"},
	{"inc", "I?      ",	2,10,"Increment index register"},
	{"inc", "(HL)    ",	1,11,"Increment indirect memory"},
	{"inc", "(I*)    ",	3,23,"Increment indirect memory"},

	{"ind", "NONE    ",	2,16,"Input to memory and decrement pointer"},
	{"indr","NONE    ",	2,21,"Input to memory and decrement pointer\n\t\t\t\tuntil byte counter is zero"},
	{"ini", "NONE    ",	2,16,"Input to memory and increment pointer"},
	{"inir","NONE    ",	2,21,"Input to memory and increment pointer\n\t\t\t\tuntil byte counter is zero"},

	{"jp",  "<addr>  ",	3,10,"Jump to location"},
	{"jp",  "<cond>,<addr>",3,10,"Jump to location if condition is satisifed"},
	{"jp",  "(HL)    ",	1, 4,"Jump to location pointed to by HL"},
	{"jp",  "(I?)    ",	2, 8,"Jump to location pointed to by I?"},

	{"jr",  "<addr>  ",	2,12,"Jump relative"},
	{"jr",  "<cond>,<addr>",2,12,"Jump relative if condition is satisfied"},

	{"ld",  "<reg>,<data>",	2, 7,"Load register with data"},
	{"ld",  "<reg>,<reg>",	1, 4,"Load source register to destination register"},
	{"ld",  "<reg>,(HL)",	1, 7,"Load register indirect from memory"},
	{"ld",  "<reg>,(I*)",	3,19,"Load register indirect from memory"},
	{"ld",  "A,I     ",	2, 9,"Move interrupt vector contents to accumulator"},
	{"ld",  "A,R     ",	2, 9,"Move refresh register contents to accumulator"},
	{"ld",  "A,(<addr>)",	3,13,"Load accumulator indirect from memory"},
	{"ld",  "A,(<rp>)",	1, 7,"Load accumulator indirect from memory by <rp>"},
	{"ld",  "I,A     ",	2, 9,"Load interrup vector from accumulator"},
	{"ld",  "R,A     ",	2, 9,"Load refresh register from accumulator"},
	{"ld",  "<rp>,<data16>",3,10,"Load register pair with data"},
	{"ld",  "<rp>,(<addr>)",4,20,"Load register pair indirect from memory"},
	{"ld",  "HL,(<addr>)",	3,20,"Load HL indirect from memory"},
	{"ld",  "I?,<data16>",	4,14,"Load I? with data"},
	{"ld",  "SP,HL   ",	1, 6,"Load contents of HL to stack pointer"},
	{"ld",  "SP,I?   ",	2,10,"Load contents of I? to stack pointer"},
	{"ld",  "(<addr>),A",	3,13,"Load contents of A to memory"},
	{"ld",  "(<addr>),HL",	3,16,"Load contents of HL to memory"},
	{"ld",  "(<addr>),<rp>",4,20,"Load contents of register pair to memory"},
	{"ld",  "(<rp>),A",	1, 7,"Load accumulator into indirect memory"},
	{"ld",  "(HL),<data>",	2,10,"Load data into indirect memory"},
	{"ld",  "(HL),<reg>",	1, 7,"Load register into indirect memory"},
	{"ld",  "(I*),<data>",	4,19,"Load data into indirect memory"},
	{"ld",  "(I*),<reg>",	3,19,"Load register into indirect memory"},

	{"ldd",	"NONE    ",	2,16,"Transfer data between memory and decrement\n\t\t\t\tdestination and source addresses"},
	{"lddr","NONE    ",	2,21,"Transfer data between memory until byte\n\t\t\t\tcounter is zero, decrement destintation and\n\t\t\t\tsource addresses"},
	{"ldi",	"NONE    ",	2,16,"Transfer data between memory and increment\n\t\t\t\tdestination and source addresses"},
	{"ldir","NONE    ",	2,21,"Transfer data between memory until byte\n\t\t\t\tcounter is zero, increment destination and\n\t\t\t\tsource addresses"},

	{"neg", "NONE    ",	2, 8,"Negate contents of accumulator"},

	{"nop",	"NONE    ",	1, 4,"No operation"},

	{"or",  "<data>  ",	2, 7,"Or data with accumulator"},
	{"or",  "<reg>   ",	1, 4,"Or register with accumulator"},
	{"or",  "(HL)    ",	1, 7,"Or indirect memory with accumulator"},
	{"or",  "(I*)    ",	3,19,"Or indirect memory with accumulator"},

	{"out", "<port>,A",	2,11,"Output from accumulator"},
	{"out", "(C),<reg>",	2,12,"Output from register"},

	{"outd","NONE    ",	2,16,"Output from memory, decrement address"},
	{"otdr","NONE    ",	2,21,"Output from memory, decrement address;\n\t\t\t\tcontinue until register B is zero"},
	{"outi","NONE    ",	2,16,"Output from memory, increment address"},
	{"otir","NONE    ",	2,21,"Output from memory, increment address;\n\t\t\t\tcontinue until register B is zero"},

	{"pop", "<rp>    ",	1,10,"Load register pair from top of stack"},
	{"pop", "I?      ",	2,14,"Load I? from top of stack"},

	{"push","<rp>    ",	1,11,"Store resister pair on top of stack"},
	{"push","I?      ",	2,15,"Store I? on top of stack"},

	{"res", "<bit>,<reg>",	2, 8,"Reset register bit"},
	{"res", "<bit>,(HL)",	2,15,"Reset bit at indirect memory location"},
	{"res", "<bit>,(I*)",	4,23,"Reset bit at indirect memory location"},

	{"ret", "NONE    ",	1,10,"Return from subroutine"},
	{"ret", "<cond>  ",	1,11,"Return from subroutine if condition true"},

	{"reti","NONE    ",	2,14,"Return from interrupt"},
	{"retn","NONE    ",	2,14,"Return from non-maskable interrupt"},

	{"rl",  "<reg>   ",	2, 8,"Rotate left through carry register contents"},
	{"rl",  "(HL)    ",	2,15,"Rotate left through carry indirect memory"},
	{"rl",  "(I*)    ",	4,23,"Rotate left through carry indirect memory"},
	{"rla", "NONE    ",	1, 4,"Rotate left through carry accumulator"},

	{"rlc", "<reg>   ",	2, 8,"Rotate left branch carry register contents"},
	{"rlc", "(HL)    ",	2,15,"Rotate left branch carry indirect memory"},
	{"rlc", "(I*)    ",	4,23,"Rotate left branch carry indirect memory"},
	{"rlca","NONE    ",	1, 4,"Rotate left accumulator"},

	{"rld",	"NONE    ",	2,18,"Rotate one BCD digit left between the\n\t\t\t\taccumulator and memory"},

	{"rr",  "<reg>   ",	2, 8,"Rotate right through carry register contents"},
	{"rr",  "(HL)    ",	2,15,"Rotate right through carry indirect memory"},
	{"rr",  "(I*)    ",	4,23,"Rotate right through carry indirect memory"},
	{"rra",	"NONE    ",	1, 4,"Rotate right through carry accumulator"},

	{"rrc", "<reg>   ",	2, 8,"Rotate right branch carry register contents"},
	{"rrc", "(HL)    ",	2,15,"Rotate right branch carry indirect memory"},
	{"rrc", "(I*)    ",	4,23,"Rotate right branch carry indirect memory"},
	{"rrca","NONE    ",	1, 4,"Rotate right branch carry accumulator"},

	{"rrd", "NONE    ",	2,18,"Rotate one BCD digit right between the\n\t\t\t\taccumulator and memory"},

	{"rst",	"NONE    ",	1,11,"Restart"},

	{"sbc", "A,<reg> ",	1, 4,"Subtract register from accumulator with borrow"},
	{"sbc", "A,<data>",	2, 7,"Subtract data from accumulator with borrow"},
	{"sbc", "A,(HL)  ",	1, 7,"Subtract indirect memory from accumulator with\n\t\t\t\tborrow"},
	{"sbc", "A,(I*)  ",	3,19,"Subtract indirect memory from accumulator with\n\t\t\t\tborrow"},
	{"sbc", "HL,<rp> ",	2,15,"Subtract register pair from HL with borrow"},

	{"scf",	"NONE    ",	1, 4,"Set carry flag"},

	{"set", "<bit>,<reg>",	2, 8,"Set register bit"},
	{"set", "<bit>,(HL)",	2,15,"Set indirect memory bit"},
	{"set", "<bit>,(I*)",	4,23,"Set indirect memory bit"},

	{"sla", "<reg>   ",	2, 8,"Shift register left arithmetic"},
	{"sla", "(HL)    ",	2,15,"Shift indirect memory left arithmetic"},
	{"sla", "(I*)    ",	4,23,"Shift indirect memory left arithmetic"},

	{"sra", "<reg>   ",	2, 8,"Shift register right arithmetic"},
	{"sra", "(HL)    ",	2,15,"Shift indirect memory right arithmetic"},
	{"sra", "(I*)    ",	4,23,"Shift indirect memory right arithmetic"},

	{"srl", "<reg>   ",	2, 8,"Shift register right logical"},
	{"srl", "(HL)    ",	2,15,"Shift indirect memory right logical"},
	{"srl", "(I*)    ",	4,23,"Shift indirect memory right logical"},

	{"sub", "<data>  ",	2, 7,"Subtract data from accumulator"},
	{"sub", "<reg>   ",	1, 4,"Subtract register from accumulator"},
	{"sub", "(HL)    ",	1, 7,"Subtract indirect memory from accumulator"},
	{"sub", "(I*)    ",	3,19,"Subtract indirect memory from accumulator"},

	{"xor", "<data>  ",	2, 7,"Exclusive or data with accumulator"},
	{"xor", "<reg>   ",	1, 4,"Exclusive or register with accumulator"},
	{"xor", "(HL)    ",	1, 7,"Exclusive or indirect memory with accumulator"},
	{"xor", "(I*)    ",	3,19,"Exclusive or indirect memory with accumulator"}};

	if (argc != 2)
	{
		printf("\nYou forgot the correct syntax?! You moron!\nUsage: define <instruction>\nNo operand needed\n");
		exit(0);
	}
		printf("\n");
		printf("OPERAND(S)\tSIZE\tCC's\tDESCRIPTION\n");
		printf("----------\t----\t----\t-----------\n");
		for(c=0;c<ENTRIES;c++)
			if (!strcmp(argv[1],dictionary[c].instruction))
				printf("%s\t%3d\t%3d\t%s\n",dictionary[c].operand,dictionary[c].size,dictionary[c].clocks,dictionary[c].description);
}